#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <string.h>

typedef unsigned char	uchar;

typedef struct {		// image data structure	 
	int 	 width;		// image width  (# cols) 
	int 	 height;	// image height (# rows) 
	uchar 	*image;		// pointer to image data 
} imageS, *imageP;

#define	BLACK		0
#define	WHITE		255
#define NEWIMAGE	((imageP) malloc(sizeof(imageS)))

imageP	readImage(char *);
imageP	allocImage(int, int);
void	saveImage(imageP, char *);
void	freeImage(imageP);
imageP	threshold(imageP, int);

void main()
{
	char	file[80];
	int		thr;
	imageP	I1, I2;

	// prompt user for filename and read image
	cout << "Image filename [try man.pgm]: ";
	cin  >>  file;
	I1 = readImage(file);

	// threshold image
	cout << "Enter threshold (0-255) [try 128]: ";
	cin  >> thr;

	// invoke threshold function
	I2 = threshold(I1, thr);

	// prompt user for output filename and save image
	cout << "Output file [try junk.pgm]: ";
	cin  >>  file;
	saveImage(I2, file);

	freeImage(I1);
	freeImage(I2);
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// threshold:
//
// Threshold image I1 and return output image I2.
// The following rule is used for thresholding:
//		v_out = 0		0   <= v_in < thr
//		v_out = 255		thr <= v_in < 256
//
imageP	
threshold(imageP I1, int thr)
{
	int		 i, w, h, total;
	uchar	*p1, *p2;
	imageP	 I2;

	// init variables
	w = I1->width;
	h = I1->height;
	total = w * h;

	// allocate I2
	I2 = allocImage(w, h);
	p1 = I1->image;
	p2 = I2->image;

	// visit all input pixels and threshold
	for(i=0; i<total; i++) {
		if(p1[i] < thr)
			 p2[i] = BLACK;
		else p2[i] = WHITE;
	}

//	more efficient: use lookup tables
//	for(i=0; i<thr; i++)	lut[i] = BLACK;
//	for(   ; i<256; i++)	lut[i] = WHITE;
//	for(i=0; i<total; i++) p[i] = lut[ p[i] ];

	return I2;

}



// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// readImage:
//
// Read image from file. The image is in PGM format.
//
imageP
readImage(char *file) 
{

	imageP	 I;
	int		 w, h, maxGray;
   	char  	 buf[80];
	uchar	*out;

	// open binary file for reading
	ifstream inFile(file, ios::binary);
	
	// error checking
	if(!inFile) {
		cerr << "readImage: Can't open " << file << endl;
		exit(1);
	}

   	// verify that the image is in PGM format. 
   	inFile.getline(buf, 3);
   	if(strncmp(buf,"P5",2) && strncmp(buf,"P6",2)){
      		cerr << "The file " << file << " is not in PGM format" << endl;
      		inFile.close();
      		exit(1);
   	}

	// read width, height, and maximum gray value
	inFile >> w >> h >> maxGray;

	// skip over linefeed and carriage return
	inFile.getline(buf, 2);

	// allocate image and read file
	I = allocImage(w, h);
	out = I->image;
	inFile.read(out, w*h);

	return (imageP) I;
}



// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// saveImage:
//
// Save image I to file in PGM format.
//
void 
saveImage (imageP I, char *file)
{

	uchar	*out;

	// open binary file for writing
	ofstream outFile(file, ios::binary);

	// error checking
	if (!outFile) {
		cerr << "saveImage: Can't open " << file << endl;
		exit(1);
	}

	// save PGM header
	outFile <<"P5" << endl;
	outFile <<I->width<< " " << I->height << endl;
	outFile << "255" << endl;

	// save image data
	out = I->image;
	outFile.write(out,I->width*I->height);
	outFile.close();

}



// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// allocImage:
//
// Allocate space for an image of width w and height h.
// Each pixel in the image has size s (in bytes).
// Return image structure pointer. 
//
imageP
allocImage(int w, int h)
{
	uchar	*p;
	imageP	 I;

	/* allocate memory */
	I = NEWIMAGE;
	p = (uchar *) malloc(w * h);
	if(p == NULL) {
		cerr << "allocImage: Insufficient memory\n";
		return ((imageP) NULL);
	}

	/* init structure */
	I->width  = w;
	I->height = h;
	I->image  = p;

	return I;
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// freeImage:
//
// Free space of image I
//
void
freeImage(imageP I)
{
	free(I->image);
	free(I);
}