#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <string.h>

typedef unsigned char	uchar;

#define	BLACK	0
#define	WHITE	255

class Image {
public:
		Image();
	int	allocImage(int, int);
	int	readImage(char *);
	int	saveImage(char *);
	uchar	getByte(int, int);
	void	setByte(int, int, uchar);
	int	getWidth();
	int	getHeight();
		~Image();
private:
	int	width;
	int	height;
	uchar	*image;
};

void threshold(Image &, int, Image &);		// function prototype

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//
// Image constructor
//
Image::Image() 
{
	width = height = 0;
	image = NULL;
}



// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//
// Image destructor
//
Image::~Image()
{
	delete[] image;
	image = NULL;
}



// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// allocImage:
//
// Allocate space for an image of width w and height h.
// Each pixel in the image has size s (in bytes).
// Return 1 for success, 0 for error (insufficient memory).
//
int Image::allocImage(int w, int h)
{
	width  = w;
	height = h;
	image = (uchar *) new uchar[w * h];

	if(image == NULL) return 0;
	return 1;
}



// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// readImage:
//
// Read image from file. The image is in PGM format.
// Return 1 for success, 0 for error (bad file open or wrong format).
//
int Image::readImage(char *file)
{

	int	maxGray;
   	char  	buf[80];

	// open binary file for reading
	ifstream inFile(file, ios::in | ios::binary);
	
	// error checking
	if(!inFile) {
		cerr << "readImage: Can't open " << file << endl;
		return 0;
	}

   	// verify that the image is in PGM format. 
   	inFile.getline(buf, 3);
   	if(strncmp(buf,"P5",2) && strncmp(buf,"P6",2)){
      		cerr << "The file " << file << " is not in PGM format" << endl;
      		inFile.close();
		return 0;
   	}

	// read width, height, and maximum gray value
	inFile >> width >> height >> maxGray;

	// skip over linefeed and carriage return
	inFile.getline(buf, 2);

	// allocate image and read file
	allocImage(width, height);
	inFile.read(image, width*height);

	return 1;	// successful read
}



// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// saveImage:
//
// Save image I to file in PGM format.
// Return 1 for success, 0 for error (bad file open).
//
 
int Image::saveImage (char *file)
{
	// open binary file for writing
	ofstream outFile(file, ios::out | ios::binary);

	// error checking
	if (!outFile) {
		cerr << "saveImage: Can't open " << file << endl;
		return 0;
	}

	// save PGM header
	outFile <<"P5" << endl;
	outFile << width << " " << height << endl;
	outFile << "255" << endl;

	// save image data
	outFile.write(image, width*height);
	outFile.close();

	return 1;	// successful save
}



// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
uchar Image::getByte(int x, int y)
{
	return image[y*width+x];
}



// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
void Image::setByte(int x, int y, uchar Byte)
{
	image[y*width+x] = Byte;
}



// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
int Image::getWidth()
{
	return width;
}



// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
int Image::getHeight()
{
	return height;
}



int main()
{
	char	file[256];
	Image	I1, I2;
	int	thr;

	// prompt user for filename and read image
	cout << "Image filename [try man.pgm]: ";
	cin  >>  file;
	
	// read input image
	I1.readImage(file);

	// prompt user for threshold value
	cout << "Enter threshold (0-255) [try 128]: ";
	cin  >> thr;

	// threshold image
	threshold(I1, thr, I2);

	// prompt user for output filename and save image
	cout << "Output file [try junk.pgm]: ";
	cin  >>  file;
	I2.saveImage(file);

	return 1;
}


// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// threshold:
//
// Threshold image I1 and return output image I2.
// The following rule is used for thresholding:
//		v_out = 0		0   <= v_in < thr
//		v_out = 255		thr <= v_in < 256
//
void threshold(Image &I1, int thr, Image &I2)
{
	int	w, h, x, y;

	// init variables
	w = I1.getWidth();
	h = I1.getHeight();

	// allocate I2
	I2.allocImage(w, h);

	// visit all input pixels and threshold
	for (y = 0; y<h; y++) {
		for(x = 0; x<w; x++)
			if (I1.getByte(x, y) > thr)
				I2.setByte(x, y, WHITE);
			else
				I2.setByte(x, y, BLACK);
	}

//	more efficient: use lookup tables
//	uchar	lut[256];
//	for(i=0; i<thr; i++)	lut[i] = BLACK;
//	for(   ; i<256; i++)	lut[i] = WHITE;
//	for (y = 0; y<h; y++) {
//		for(x = 0; x<w; x++) {
//			I2.setByte(x, y, lut[I1.getByte(x, y)]);
//		}
//	}

}
